// CycloneDriver.h: interface for the CCycloneDriver class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CYCLONEDRIVER_H__39DCA5E2_9CB7_4D41_9141_A32214B04A1A__INCLUDED_)
#define AFX_CYCLONEDRIVER_H__39DCA5E2_9CB7_4D41_9141_A32214B04A1A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define NOTIFYBUFFER_SIZE		0x4000   //10K
#define DRIVER_EVENT_NAME "Semaphore{2956497D-A685-433e-9F25-267FA3B0E715}"
 
typedef struct _NOTIFYBUFFER
{
	long count;							// Amount of data currently in buffer
	long write;							// write count - this increments but is never reset
	long read;							// read point - this is only changed by sysman
	long size;							// how many entries? This equals RINGBUFFER_ENTRIES and never changes
	char buffer[NOTIFYBUFFER_SIZE];		// Our actual entries

} NOTIFYBUFFER;

typedef struct _SHARED_BUFFER
{
	unsigned long		ulSig;			// 0xbeefdead
	NOTIFYBUFFER		notify_buffer;

} SHARED_BUFFER, *PSHARED_BUFFER;


class CNotification;
class CCycloneDriver  : public CMclThreadHandler
{
public:
	int OpenDriver(CNotification* p);
	void CloseDriver();

	bool EnableNotify();
	bool DisableNotify();

	static CCycloneDriver& Instance();
		
	virtual ~CCycloneDriver();
	bool IsOpen() {	return INVALID_HANDLE_VALUE!=m_hDevice;}

	static bool ReadOneRingBufferEntry(	NOTIFYBUFFER& ring_buffer, 
										std::vector<char>& entry);

	unsigned ThreadHandlerProc(void);


	operator bool () {return m_bValid;}

private:

	void StopNotifyThread()
	{
		m_EventStop.Set();
		m_EventStopAck.Wait(INFINITE);
	}
	bool AcquireShm();
	const static char* g_LinkName;
	CCycloneDriver();
 

	bool m_bValid;
	HANDLE m_hDevice;
	CNotification* m_pNotify;

	PVOID m_pShm;

	//
	// The sempaphore to  get the notification event
	//
	HANDLE m_hSemEvtAvail;

	CMclEvent m_EventStop;
	CMclEvent m_EventStopAck;
	HANDLE m_NotifyAvail;

	bool m_bNotifyThreadStartup;
  
};

struct MemoryResult
{
	PVOID m_ret;
	ULONG m_size;
};

#endif // !defined(AFX_CYCLONEDRIVER_H__39DCA5E2_9CB7_4D41_9141_A32214B04A1A__INCLUDED_)
