// UserMemory.cpp: implementation of the CUserMemory class.
//
//////////////////////////////////////////////////////////////////////
#include <vdw.h>
#include "UserMemory.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CUserMemory* CUserMemory::m_pInstance=(CUserMemory*)0;

CUserMemory* CUserMemory::Instance()
{
	if (!m_pInstance)
		m_pInstance= new (NonPagedPool) CUserMemory;
	return m_pInstance;	
}

void CUserMemory::Cleanup()
{
	if (m_pInstance)
	{
		delete m_pInstance;
	}
}

CUserMemory::CUserMemory() 
{
	m_pBuffer= new (NonPagedPool) CHAR[MAX_BUFFER_SIZE];
	PSHARED_BUFFER p=(PSHARED_BUFFER)m_pBuffer;

	memset(m_pBuffer,0,MAX_BUFFER_SIZE);
	p->ulSig=0xbeefdead;
	p->notify_buffer.size=NOTIFYBUFFER_SIZE;
	
	m_pMdl= new (NonPagedPool) KMemory(m_pBuffer,MAX_BUFFER_SIZE);

	m_pNotifyEvent= new (NonPagedPool) KEvent(KUstring(DRIVER_EVENT_NAME),SynchronizationEvent);
	m_pNotifyEvent->Clear();
}

CUserMemory::~CUserMemory()
{
	if (m_pMdl)
	{
		delete m_pMdl;
	}
	if (m_pBuffer)
	{
		delete m_pBuffer;
	}

	delete m_pNotifyEvent;
}

PVOID CUserMemory::MapToUserSpace()
{
	return m_pMdl->MapToUserSpace();
}

void CUserMemory::Unmap(PVOID p)
{
	m_pMdl->Unmap(p);
}
  

bool CUserMemory::WriteToRingBuffer( NOTIFYBUFFER & ring_buffer, char * data, int datasize )
{
	if( !data || datasize <= 0 )
		return false;
	
	ULONG	dwNewLen = 0;
	ULONG	size = 0;
	ULONG	write_location = 0;
	ULONG	* size_field = NULL;
	ULONG	first_chunk_size = 0;

	dwNewLen = datasize + 3 & ~3;
	size = dwNewLen + sizeof(ULONG);

	if( InterlockedExchangeAdd( &ring_buffer.count, size ) + (long)size > ring_buffer.size )
	{
		InterlockedExchangeAdd( &ring_buffer.count, -(long)size );
		return false;
	}

	write_location = InterlockedExchangeAdd( &ring_buffer.write, size );
	write_location %= ring_buffer.size;
	size_field = (ULONG *)( ring_buffer.buffer + write_location );
	ASSERT( *size_field == 0 );

	write_location = (write_location + sizeof(ULONG)) % ring_buffer.size;
	if( write_location + dwNewLen > (ULONG)(ring_buffer.size) )
	{
		first_chunk_size = ring_buffer.size - write_location;
		memcpy( ring_buffer.buffer + write_location, data, first_chunk_size );
		memcpy( ring_buffer.buffer, data + first_chunk_size, datasize - first_chunk_size );
	}
	else
	{
		memcpy( ring_buffer.buffer + write_location, data, datasize );
	}

	*size_field = dwNewLen;
	return true;
}

void CUserMemory::NotifyWrite()
{
	BOOLEAN statu=m_pNotifyEvent->Set();
	m_pNotifyEvent->Clear();
}
